var Class = require("core").Class;
var Types = require("core").Common.Types;
var CdfTag = require("./CdfTag.js").CdfTag;

/**
 * @class CdfModel
 * @extends CdfTag
 */
var CdfModel = function(){
    this.Extends = CdfTag;
    
    
    this.contentType = null;
    this.cache = null;  // czy tutaj nie powinna byc domyslna wartosc???
    this.onError = null;
    this.target = null;
    
    this._body = null;
    this._renderedBody = null;
    this._isRendered = false;
    
    
    this.initialize = function(data){
        this.parent(data);
        if(data){
            this._parse();
        }
    };
    
    this._parse = function(){
        
        var matches = this.token.match(/^<cdf:model([^>]+)><!\[CDATA\[((.|\s)*?)\]\]><\/cdf:model>$/i);
        
        var attrMatches = matches[1].match(/\S+="\S+?"/ig);
        
        var match = null;
        var valuePair = null;
        
        for(var i=0,max=attrMatches.length;i<max;i++){
            match = attrMatches[i];
            valuePair = match.match(/(\S+)="(\S+?)"/i);
            this._parseParam(valuePair[1], valuePair[2]);
        }
        
        this.setBody(matches[2]);
        
        if(!this.isRenderable()){
            this.setRenderedData(this.getBody());
        }
    };
    
    this._parseParam = function(name, value){

        switch(name){
            
            case "content-type":
                this.contentType = value;
            break;
            
            case "cache":
                this.cache = value;
            break;
            
            case "onError":
                this.onError = value;
            break;
            
            case "target":
                this.target = value;
            break;
            
            default:
                console.log('ERROR: CdfModel::_parseParam : unsupported param name: '+name);
            break;
        }
    };
    
    
    this.setBody = function(data){
        if(Buffer.isBuffer(data)){
          this._body = data;
        } else if(Types.isString(data)){
          this._body = data;
        } else {
          this._body = '<![CDATA[\n' + JSON.stringify(data) + '\n]]>';
        }        
    };
    /**
     * Returns orginal body
     */
    this.getBody = function(){
        return this._body;
    };
    
    //Zastanawiam sie nad podejsciem setBody("dane", contentType)
    this.setRenderedData = function(data){
        this._isRendered = true;
        this._renderedBody = data;
    };
    
    this.getRenderedData = function(){
        return this._renderedBody;
    };
    
    this.isRendered = function(){
        return this._isRendered;
    };
    
    this.isCacheable = function(){
        // zakladam ze zawsze jest domyslna wartosc
        return this.cache != CdfModel.Cache.Label.NONE
    };
    
    this.getCacheTime = function(){
        if(this.cache==CdfModel.Cache.Label.NONE){
            return CdfModel.Cache.Value.NONE;
        
        } else if(this.cache==CdfModel.Cache.Label.MIN){
            return CdfModel.Cache.Value.MIN;
        
        } else if(this.cache==CdfModel.Cache.Label.MEDIUM){
            return CdfModel.Cache.Value.MEDIUM;
        
        } else if(this.cache==CdfModel.Cache.Label.MAX){
            return CdfModel.Cache.Value.MAX;
        
        } else {
            console.log('        -- WARNING: inproper CDF cache value: '+this.cache);
            return CdfModel.Cache.Value.NONE;
        }
    };
    
    this.isRenderable = function(){
        return this.contentType == 'application/json';
    }
    

    /**
     * Zwraca kompletnego stringa cdf <cdf:
     */
    this.toString = function(){
        var arrVal = [];
        if(this.target){
            arrVal.push('target="' + this.target + '"');
        }
        if(this.contentType){
            arrVal.push('content-type="' + this.contentType + '"');
        }
        if(this.cache){
            arrVal.push('cache="' + this.cache + '"');
        }
        if(this.onError){
            arrVal.push('onError="' + this.onError + '"');
        }
        return "<cdf:model " + arrVal.join(" ") + ">"+this._body+"</cdf:model>";
    };
};

CdfModel = new Class( new CdfModel() );

CdfModel.onError = {};

CdfModel.onError.ABORT ="abort";
CdfModel.onError.DISCARD ="discard";
CdfModel.onError.PROPAGATE ="propagete"; 

CdfModel.Cache = {};

CdfModel.Cache.Label = {};
CdfModel.Cache.Label.NONE   = 'none';
CdfModel.Cache.Label.MIN    = 'min';
CdfModel.Cache.Label.MEDIUM = 'mdm';
CdfModel.Cache.Label.MAX    = 'max';

CdfModel.Cache.Value = {};
CdfModel.Cache.Value.NONE   = 0;
CdfModel.Cache.Value.MIN    = 90;
CdfModel.Cache.Value.MEDIUM = 300;
CdfModel.Cache.Value.MAX    = 18000;

exports.CdfModel = CdfModel;
