var Class = require("core").Class;
var CdfContainer = require('./CdfContainer.js').CdfContainer;
var CdfModel = require('./CdfModel.js').CdfModel;

/**
 * @class CdfModels
 */
var CdfModels = function(){

    this.Extends = CdfContainer;


    /**
     * @ignore
     */ 
    this.initialize = function(){
        this.parent();
    };
    
    this.add = function(model){
        this.parent(model);
    };
    
    this.remove = function(model){
        //this.  
    };
    
    /**
     * Określa czy isteją modele do zrenderowania
     * @returns {Boolean}
     */
    this.hasRenderableModels = function(){
        for(var i=0,max=this._models.length;i<max;i++){
            if(this._models[i].isRendered()==false){
                return true;
            }
        }
        return false;
    };
    
    this.hasNonCacheableModels = function(){
        for(var i=0,max=this._models.length;i<max;i++){
            if(this._models[i].isCacheable()==false){
                return true;
            }
        }
        return false;
    };
    
    this.hasCacheableModels = function(){
        for(var i=0,max=this._models.length;i<max;i++){
            if(this._models[i].isCacheable()){
                return true;
            }
        }
        return false;
    };
    
    /**
     * Zwraca modele mozliwe do umieszczenia w pamieci podrecznej (cache)
     * @returns {Array}
     */
    this.getCacheableModels = function(){
        var models = [];
        for(var i=0,max=this._models.length;i<max;i++){
            this._models[i].isCacheable() && models.push(this._models[i]);
        }
        return models;
    };
    /**
     * @returns {Array}
     */
    this.getNonCacheableModels = function(){
        var models = [];
        for(var i=0,max=this._models.length;i<max;i++){
            !this._models[i].isCacheable() && models.push(this._models[i]);
        }
        return models;
    };
    
    /**
     * Zwraca modele nadające sie do zrenderowania
     * @returns {Array}
     */
    this.getRenderableModels = function(){
        var models = [];
        for(var i=0,max=this._models.length;i<max;i++){
            !this._models[i].isRendered() && models.push(this._models[i]);
        }
        return models;
    };
    
    this.hasTarget = function(id){
        for(var i=0,max=this._models.length;i<max;i++){
            if(this._models[i].target==id){
                return true;
            }
        }
        return false;
    };
    
    this.getSmallestCacheTime = function(){
        var smallestCacheTime = CdfModel.Cache.Value.MAX;
        var tmpCacheTime = null;
        
        for(var i=0,max=this._models.length;i<max;i++){
            tmpCacheTime = this._models[i].getCacheTime();
            if(tmpCacheTime<smallestCacheTime && tmpCacheTime>0){
                smallestCacheTime = tmpCacheTime;
            }
        }
        return smallestCacheTime;
        
    };
};

CdfModels = new Class( new CdfModels() );
exports.CdfModels = CdfModels;
