var Class = require("core").Class;
var Event           = require("core").Event.Event;
var ErrorEvent      = require("core").Event.ErrorEvent;
var EventDispatcher  = require("core").Event.EventDispatcher;
var CdfModel = require('./CdfModel.js').CdfModel;
var CdfTarget = require('./CdfTarget.js').CdfTarget;
var CdfModels = require('./CdfModels.js').CdfModels;
var CdfTargets = require('./CdfTargets.js').CdfTargets;

var CDF_TAG_PATTERN = /(<cdf:(.|\s)+?(<\/cdf:[^>]+>))/ig
var CDF_TARGET_PATTERN = /(<cdf:target(.|\s)+?(<\/cdf:[^>]+>))/ig;
var CDF_MODEL_PATTERN = /(<cdf:model(.|\s)+?(<\/cdf:[^>]+>))/ig

var CdfPage = function(){
    this.Extends = EventDispatcher;
    this._data = "";
    
    this.targets = null;
    this.models = null;
    
    this.layout = null;
    
    this.cdftargetPattern = null;
    this.cdfmodelPattern = null;
    
    /**
     * Parsuje zadanego stringa w poszukiwaniu cdf:mode i cdf:target 
     * @ignore
     */
    this.initialize = function(data){
        this.cdftargetPattern = CDF_TARGET_PATTERN
        this.cdfmodelPattern = CDF_TARGET_PATTERN
        
        this.targets = new CdfTargets();
        this.models = new CdfModels();
        
        this._data = data;

        var hasSkeleton = false;

        //czy mam szkielet
        if(data.search("target=\".skeleton\"") > -1){
            hasSkeleton = true;
            this._data = "";
        }
        
        //parse data
        var chunks = data.match(CDF_MODEL_PATTERN);
        
        
        if(chunks){
            var model, chunk = null;
            for(var i = 0, max = chunks.length; i < max; i++){
                chunk = chunks[i];
                model = new CdfModel(chunk);
                this.models.add( model );
                if(hasSkeleton){
                    if(model.target == ".skeleton"){
                        this.layout = model;
                    }
                    this._data += model.token;    
                }   
            }
        }
        
    };
    
    this.render = function(){
        
        console.log('     -- CDF Page Render:');
        
        var body = this._data;
        
        if(this.layout){
            console.log('        -- I HAVE special layout model');
            if(!this.layout.isCacheable()){
                this.dispatchEvent( new Event(CdfPage.Event.CACHE_REDY, body));
            } else{
                body = body.replace(this.layout.token, this.layout.getRenderedData()+"\n\n");
            }
        } else {
            console.log('        -- I DONT HAVE special layout model');
        }
        
        
        
        console.log('        -- Finding targets...');
        var target = null, chunk = "", chunks = body.match( this.cdftargetPattern );
        if(chunks){
            for(var i=0,max=chunks.length;i<max;i++){
                chunk = chunks[i];
                target = new CdfTarget(chunk);
                if(this.models.hasTarget(target.id)){
                    
                    this.targets.add(new CdfTarget(chunk));    
                }
                else{
                    body.replace(chunk, "");
                }
            }            
        }
        console.log('        -- Targets found: ' + this.targets.getLength());
        
        if(this.models.hasCacheableModels()){
            console.log('        -- Replacing CACHEABLE models...');
            body = this._replaceCdf(body, this.models.getCacheableModels() );
            this.dispatchEvent(new Event(CdfPage.Event.CACHE_READY, body));
        } else {
            console.log('        -- There is no CACHABLE models');
        }
        
        console.log('        -- Replacing NONCACHEABLE models...');
        body = this._replaceCdf(body, this.models.getNonCacheableModels());
        this.dispatchEvent(new Event(CdfPage.Event.RENDERED, body));
        
    };
    
    this._replaceCdf = function(body, models){
        console.log('           -- replacing '+models.length+" models");
        if(models){
            var model = null, target = null;
        
            for(var i=0, max=models.length; i<max;i++){
                model = models[i];
                if(model.target){
                    target = this.targets.getById(model.target);
                    if(target){
                        body = body.replace(model.token, '');
                        body = body.replace(target.token, model.getRenderedData());
                    } else {
                        //FIXME:
                        console.log('           -- WARNING: Cannot find target: "'+model.target+'" for model');
                    }
                } else {
                    body = body.replace(model.token, model.getRenderedData());
                }
            }
        }
        console.log('           -- replace CDF finished!');
        return body;
    };
}

CdfPage = new Class( new CdfPage() );

CdfPage.Event = {};
CdfPage.Event.RENDERED = "PAGE_RENDERED";
CdfPage.Event.CACHE_REDY = "PAGE_CACHE_REDY";
CdfPage.Event.ERROR = "PAGE_ERROR";
exports.CdfPage = CdfPage;