var Core = require("core");
var Class = require("core").Class;
var DriverAbstract = require("./DriverAbstract.js").DriverAbstract;
var Event = require('core').Event.Event;
var ErrorEvent = require('core').Event.ErrorEvent;
var Crypto = require('core').Common.Crypto;
var knox = require('knox');

/**
 * @class S3
 */
var S3 = function(){
    this.Extends = DriverAbstract;
    
    this._application = null;
    this._credentialKey = null;
    
    this._client = null;
    this._credentialData = null;
    
    this.initialize = function(application, configuration){
        this._application = application;
        this._configuration = configuration;
        
        this._client = knox.createClient({
            key:    this._configuration.ACCESS_KEY,
            secret: this._configuration.SECRET_KEY,
            bucket: this._configuration.buckets.BUCKET_TEMP
        });
    };
    
    this.set = function(data){
        var that = this;
        var req = this._client.put(Crypto.md5(data), {
            'Content-Length': data.length,
            'Content-Type': 'application/octet-stream'
        });
        req.on('response', function(res){
            if(200 == res.statusCode){
                that.registerUrl(req.url);
            } else {
                that.dispatchEvent(new ErrorEvent(DriverAbstract.Event.ERROR, {res: res, req:req}, -5, 'S3 Error'));
            }
        });
        req.end(data);
    };
    
};

S3 = new Class( new S3() );
exports.S3 = S3;